/*

   Programa de lectura de un directorio remoto en base a 
   sockets de la familia Unix. 
   Codigo del cliente con time out, si despues de 25 segundos
   el servidor no responde deja de insistir.
   Es necesario que el servicio "lectura_directorio"
   este dado de alta en el archivo "/etc/services".
 
   Nombre Archivo: udpcltsvc.c
   Archivos relacionados: udpcltsvc.c lee_client.c
   Fecha: octubre 97

   Compilacion: cc udpcltsvc.c -lnsl -lsocket -o udpcltsvc 

   Ejecucion: udpcltsvc <host> <puerto> <directorio>

*/


#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>

#define  SERVICIO_UDP  	"lectura_directorio"   /* nombre del servicio */
#define  DIRSIZE	2048                   /* longitud maxima parametro entrada/salida */

main(argc, argv)
   int argc;  
   char *argv[];
{	

	char  		      dir[DIRSIZE];  /* parametro entrada y salida */
        char                  *host;         /* nombre del host */
	int                   sd;            /* descriptor del socket */
        int                   addrlen;       /* longitud direccion */
        int                   retry;         /* contador intentos conexion */
	struct sockaddr_in    myaddr;	     /* descriptor internet */
	struct sockaddr_in    svaddr;	     /* descriptor servidor */
	struct sockaddr_in    claddr; 	     /* direccion internet del cliente */
	struct hostent	      *hp;	     /* estructura descripcion host */
	struct servent	      *sp;	     /* estructura descripcion servicios */


	
/* sirve de algo    */

	void  t_nop()
	{
		fprintf(stderr, "esperando por un servicio....\n");
		signal(SIGALRM, t_nop);
	}


/* verificar los parametros */
/* Nota: no se pide puerto ya que este se obtiene a partir de getservbyname() */

        if (argc !=3 )  {
            fprintf(stderr,"Error, uso: %s <host> <directorio> \n",argv[0]);
            exit(1);
        }
        host =argv[1];

/* llenar la estructura de direcciones con la informacion del host */

	svaddr.sin_family = AF_INET;

        /* obtencion de los datos del host */

	hp = gethostbyname(host);
	if (hp == NULL) {
		fprintf(stderr,"No se encontro %s en /etc/hosts \n",host);
		exit(1);
	}
	svaddr.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;

        /* obtencion de los datos del servicio (numero puerto) */

	sp = getservbyname(SERVICIO_UDP, "udp");
	if (sp == NULL) {
		fprintf(stderr, "No se encontro %s en /etc/hosts \n",SERVICIO_UDP);
		exit(1);
	}
	svaddr.sin_port = sp->s_port;

/* obtencion de un socket tipo internet */

	if ( (sd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

/* anadiendo informacion sobre el host local (cliente) */

	myaddr.sin_family = AF_INET;
	myaddr.sin_port = 0;
	myaddr.sin_addr.s_addr = INADDR_ANY;

/* asociado el socket al puerto  */

	if (bind(sd,  (struct sockaddr *)&myaddr, sizeof( struct sockaddr_in)) == - 1) {
		perror("bind");
		exit(1);
	}

/* intenta enviar un mensaje al servidor */

	signal(SIGALRM, t_nop);
	retry = 5;
	addrlen = sizeof(svaddr);
again:	if (sendto(sd, argv[2], strlen(argv[2]), 0, (struct sockaddr *)&svaddr, addrlen) == -1) {
		perror("sendto");
		exit(1);
	}
	alarm(5);			/* signal SIGALRM es repetida cada 5 segundos */
	if  (recvfrom(sd, dir, DIRSIZE, 0, (struct sockaddr *)&claddr, &addrlen) == -1) {
		if (errno == EINTR) {
			if  (--retry) 
				goto again;
			else{
			        fprintf(stderr,"Abortando, servidor no respondio \n");	
				exit(1);
			}
		}
		else {
		        perror("recvfrom");
		        exit(1);
		}
	}
	alarm(0);         /*resetear la alarma */	

/* en caso de exito: se imprimen resultados y se cierra todo */

	printf("%s \n", dir);
	close(sd);
}
