/*

   Programa de lectura de un directorio remoto en base a 
   sockets de la familia Unix. 
   Codigo del cliente con time out, si despues de 25 segundos
   el servidor no responde deja de insistir.
 
   Nombre Archivo: udptimeout.c
   Archivos relacionados: udpserver.c lee_client.c
   Fecha: octubre 97

   Compilacion: cc udptimeout.c -lnsl -lsocket -o timeout

   Ejecucion: timeout <host> <puerto> <directorio>

*/


#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>

#define  DIRSIZE     2048    /* longitud parametro entrada/salida */

main(argc, argv)
   int argc;  
   char *argv[];
{	

	char  		      dir[DIRSIZE];  /* parametro entrada y salida */
        char                  *host;         /* nombre del host */
        int	              puerto;	     /* numero de puerto */
	int                   sd;            /* descriptor del socket */
        int                   addrlen;       /* longitud direccion */
        int                   retry;         /* contador intentos conexion */
	struct sockaddr_in    myaddr;	     /* descriptor internet */
	struct sockaddr_in    svaddr;	     /* descriptor servidor */
	struct sockaddr_in    claddr; 	     /* direccion internet del cliente */
	struct hostent	      *hp;	     /* estructura descripcion host */

	
/* sirve de algo    */

	void  t_nop()
	{
	  /* No hace nada solo reactiva la signal SIGALARM */ 
           signal(SIGALRM, t_nop);
	}


/* verificar los parametros */

        if (argc != 4)  {
            fprintf(stderr,"Error, uso: %s <host> <puerto> <directorio> \n",argv[0]);
            exit(1);
        }
        host =argv[1];
        puerto = atoi(argv[2]);

/* llenar la estructura de direcciones con la informacion del host */

	svaddr.sin_family = AF_INET;
	hp = gethostbyname(host);
	if (hp == NULL) {
		fprintf(stderr,"No se encontro %s en /etc/hosts \n",host);
		exit(1);
	}
	svaddr.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
	svaddr.sin_port = htons(puerto);
	
/* obtencion de un socket tipo internet */

	if ( (sd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

/* anadiendo informacion sobre el cliente */

	myaddr.sin_family = AF_INET;
	myaddr.sin_port = puerto;
	myaddr.sin_addr.s_addr = INADDR_ANY;

/* asociando el socket al puerto  */

	if (bind(sd, (struct sockaddr *)&myaddr, sizeof( struct sockaddr_in)) == - 1) {
		perror("bind");
		exit(1);
	}

/* intenta enviar un mensaje al servidor */

	signal(SIGALRM, t_nop);
	retry = 5;
	addrlen = sizeof(svaddr);
again:	if ( sendto(sd, argv[3], strlen(argv[3]), 0, (struct sockaddr *)&svaddr, addrlen) == -1) {
		perror("sendto");
		exit(1);
	}
	alarm(5);			/* signal SIGALRM es repetida cada 5 segundos */
        printf("Se envia peticion, intento no. %d \n",6-retry);
	if  ( recvfrom(sd, dir, sizeof(dir), 0, (struct sockaddr *)&claddr, &addrlen) == -1) {
		if (errno == EINTR) {
			if  (--retry) { 
				goto again;
			} else {
				fprintf(stderr,"Servidor: %s no respondio... abortando\n",host);
				exit(1);
			}
		}
		else {
		        perror("recvfrom");
		        exit(1);
		}
	}
	alarm(0);         /*resetear la alarma */	

/* en caso de exito: se imprimen resultados y se cierra todo */

	printf("\n%s \n", dir);

	close(sd);
}
